package com.me.asteroids.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.me.asteroids.Constants;
import com.me.asteroids.EntityFactory;
import com.me.asteroids.Graphics;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.BulletComponent;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PlayerComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.asteroids.events.ScreenWrapEvent;
import com.me.asteroids.systems.CollisionSystem;
import com.me.asteroids.systems.ModelRenderSystem;
import com.me.asteroids.systems.MovementSystem;
import com.me.asteroids.systems.PlayerInputSystem;
import com.me.asteroids.systems.ScreenWrapSystem;
import com.me.common.Screen;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.event.EventHandler;
import com.me.common.ecs.event.Listener;

public class GameScreen extends Screen implements Listener {

    Engine engine;

    Graphics graphics;

    SpriteBatch batch;
    BitmapFont font;

    public GameScreen(Graphics graphics) {
        this.graphics = graphics;
    }

    @Override
    public void setup() {
        batch = new SpriteBatch();
        font = new BitmapFont();
        font.setColor(Color.RED);

        engine = new Engine();

        engine.registerComponentClass(PlayerComponent.class);
        engine.registerComponentClass(BulletComponent.class);
        engine.registerComponentClass(PositionComponent.class);
        engine.registerComponentClass(VelocityComponent.class);
        engine.registerComponentClass(AccelerationComponent.class);
        engine.registerComponentClass(ModelComponent.class);

        engine.registerSystem(new PlayerInputSystem(engine));
        engine.registerSystem(new MovementSystem(engine));
        engine.registerSystem(new CollisionSystem(engine));
        engine.registerSystem(new ScreenWrapSystem(engine));
        engine.registerSystem(new ModelRenderSystem(engine, graphics));

        engine.registerListener(this);

        engine.ready();

        Entity player = EntityFactory.createPlayer(engine);
        player.activate();
    }

    @Override
    public void update(float dt) {
        graphics.reset();
        engine.update(dt);

        if (Constants.DEBUG) {
            batch.begin();
            font.draw(batch, String.format("FPS: %d, Entities: %d", Gdx.graphics.getFramesPerSecond(), engine.getEntityCount()), 50, 50);
            batch.end();
        }
    }

    @Override
    public void dispose() {

    }

    @EventHandler
    public void onScreenWrap(ScreenWrapEvent event) {
        if (event.entity.hasComponent(BulletComponent.class)) {
            // Remove bullets when they leave the screen
            event.setCancelled(true);
            event.entity.remove();
        }
    }


}
