package com.me.common.ecs;

public final class Entity {

    private static int nextId = 0;
    private static long nextUniqueId = 0;

    private Engine engine;

    protected int id;
    protected long componentBits;

    private long uniqueId;
    private boolean active;

    protected Entity(Engine engine) {
        this.engine = engine;
        this.active = false;
        this.id = nextId++;
    }

    public int getId() {
        return id;
    }

    public long getUniqueId() {
        return uniqueId;
    }

    public boolean isActive() {
        return active;
    }

    public void activate() {
        active = true;
        engine.activateEntity(this);
    }

    public void deactivate() {
        active = false;
        engine.deactivateEntity(this);
    }

    public void remove() {
        engine.removeEntity(this);
    }

    public Engine getEngine() {
        return this.engine;
    }

    public boolean hasComponent(Class<? extends Component> clazz) {
        return ComponentType.getComponentType(clazz).isTypeInMask(componentBits);
    }

    public <T extends Component> T getComponent(Class<T> clazz) {
        return engine.getEntityComponent(this, clazz);
    }

    public void addComponent(Component component) {
        engine.addEntityComponent(this, component);
    }

    public void removeComponent(Component component) {
        engine.removeEntityComponent(this, component);
    }

    protected void addComponentType(ComponentType type) {
        componentBits |= type.bits;
    }

    protected void removeComponentType(ComponentType type) {
        componentBits &= ~type.bits;
    }

    protected void reset() {
        componentBits = 0;
        active = false;
    }

    protected void updateUniqueId() {
        this.uniqueId = nextUniqueId++;
    }

}
